@@ -716,7 +716,7 @@ int get_mask(int num_bits)
 }
 
 void __attribute__((__sentinel__(0)))
-radeon_show_reg_bits(const char *name, unsigned long index_addr,
+radeon_show_reg_bits(const char *prefix, const char *name, unsigned long index_addr,
                      unsigned long data_addr, unsigned long addr, ...)
 {
     va_list ap;
@@ -730,7 +730,7 @@ radeon_show_reg_bits(const char *name, unsigned long index_addr,
     else
         value = radeon_get(addr, name);
 
-    printf("%s\t%08x\n", name, value);
+    printf("%s%s\t%08x\n", prefix ? prefix : "", name, value);
 
     va_start(ap, addr);
     while (1) {
@@ -915,19 +915,178 @@ const char *get_reg_name(unsigned long address, const char *type)
     return NULL;
 }
 
+#define GET_REG(r) radeon_get(r, #r)
+#define SHOW_REG(r) printf("%s\t%08x\n", #r, radeon_get(r, #r))
+#define SHOW_UNKNOWN_REG(r) { tmp = radeon_get(r, "byhand"); printf("%08lx\t%08x (%d)\n", r, tmp, tmp); }
+#define REG_TYPE_NAME(r) ((r == REG_TYPE_STATIC) ? "static" : ((r == REG_TYPE_SEMI_STATIC) ? "semi-static" : "random"))
+#define SHOW_STATIC_REG(r) { tmp = get_reg_type(r); printf("%s (%08lx)\t%s\n", get_reg_name(r, ""), r, REG_TYPE_NAME(tmp)); }
+#define SHOW_REG_DECIMAL(r) printf("%s\t%d (decimal)\n", #r, radeon_get(r, #r))
+#define SHOW_REG_BITS(r, ...) radeon_show_reg_bits(NULL, #r, 0, 0, r, __VA_ARGS__)
+#define SHOW_MC_REG(r) printf("%s\t%08x\n", #r, radeon_get_indexed(AVIVO_MC_INDEX, AVIVO_MC_DATA, (r | 0x007f0000), #r))
+#define SHOW_MC_REG_BITS(r, ...) radeon_show_reg_bits(NULL, AVIVO_MC_INDEX, AVIVO_MC_DATA, #r, (r | 0x007f0000), __VA_ARGS__)
+#define SHOW_PCIE_REG(r) printf("%s\t%08x\n", #r, radeon_get_indexed(0x30, 0x34, (r), #r))
+
+static uint32_t eg_offsets[] = { EVERGREEN_CRTC0_REGISTER_OFFSET, EVERGREEN_CRTC1_REGISTER_OFFSET,
+			     EVERGREEN_CRTC2_REGISTER_OFFSET, EVERGREEN_CRTC3_REGISTER_OFFSET,
+			     EVERGREEN_CRTC4_REGISTER_OFFSET, EVERGREEN_CRTC5_REGISTER_OFFSET };
+
+#define EG_NUM_OFFSETS (sizeof(eg_offsets) / sizeof(eg_offsets[0]))
+
+#define EG_GET_MODE_REG(index, reg) GET_REG((reg) + eg_offsets[(index)])
+void eg_cmd_regs(const char *type)
+{
+    int show_all = (strcmp(type, "all") == 0);
+    int show_grphs = 0;
+    int i;
+    uint32_t tmp, tmp1;
+    char tmpname[10];
+    
+    for (i = 0; i < EG_NUM_OFFSETS; i++) {
+	tmp = EG_GET_MODE_REG(i, EVERGREEN_CRTC_CONTROL);
+	tmp1 = EG_GET_MODE_REG(i, EVERGREEN_CUR_CONTROL);
+
+	if (tmp & 0x1)
+	    show_grphs |= (1 << i);
+
+	printf("D%dCRTC: %s, Cursor %s\n", i+1, tmp & 0x1 ? "Enabled" : "Disabled",
+	       tmp1 & 0x1 ? "Enabled" : "Disabled");
+	if (!(tmp & 0x1))
+	    continue;
+
+	tmp = EG_GET_MODE_REG(i, EVERGREEN_VIEWPORT_START);
+	tmp1 = EG_GET_MODE_REG(i, EVERGREEN_VIEWPORT_SIZE);
+	printf("Viewport start x: %d y: %d, w %d h %d\n", tmp >> 16, tmp & 0xffff,
+	       tmp1 >> 16, tmp1 & 0xffff);
+	    
+    }
+
+    for (i = 0; i < EG_NUM_OFFSETS; i++) {
+
+	if (!(show_grphs & (1 << i)))
+	    continue;
+
+	snprintf(tmpname, 10, "D%dGRPH", i+1);
+	tmp = EG_GET_MODE_REG(i, EVERGREEN_GRPH_ENABLE);
+	if (!(tmp & 0x1)) {
+	    printf("\n%s: disabled\n", tmpname, i);
+	    continue;
+	}
+	printf("\n%s: enabled %08x, control %08x\n", tmpname, tmp,
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_CONTROL));
+	radeon_show_reg_bits(tmpname, "CONTROL", 0, 0,
+			     EVERGREEN_GRPH_CONTROL + eg_offsets[i],
+			     0, 1, "Depth",
+			     8, 10, "Format", 
+			     0, 0, NULL);
+	printf("pitch %08x\n",
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_PITCH));
+	printf("Primary surface address %08x:%08x\n",
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_PRIMARY_SURFACE_ADDRESS_HIGH),
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_PRIMARY_SURFACE_ADDRESS));
+	printf("Secondary surface address %08x:%08x\n",
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_SECONDARY_SURFACE_ADDRESS_HIGH),
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_SECONDARY_SURFACE_ADDRESS));
+	printf("Surface offset X %08x Y %08x\n",
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_SURFACE_OFFSET_X),
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_SURFACE_OFFSET_Y));
+	printf("X/Y start %08x %08x, end %08x %08x\n",
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_X_START),
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_Y_START),
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_X_END),
+	       EG_GET_MODE_REG(i, EVERGREEN_GRPH_Y_END));
+    }
+
+    printf("\n");
+    for (i = 0; i < EG_NUM_OFFSETS; i++) {
+	tmp = EG_GET_MODE_REG(i, EVERGREEN_CUR_CONTROL);
+	if (!(tmp & 0x1))
+	    continue;
+
+	snprintf(tmpname, 10, "D%dCUR", i+1);
+	radeon_show_reg_bits(tmpname, "CONTROL", 0, 0,
+			     EVERGREEN_CUR_CONTROL,
+			     0, 0, "Enable",
+			     8, 10, "Mode",
+			     0, 0, NULL);
+	printf("Cursor surface address %08x:%08x\n",
+	       EG_GET_MODE_REG(i, EVERGREEN_CUR_SURFACE_ADDRESS_HIGH),
+	       EG_GET_MODE_REG(i, EVERGREEN_CUR_SURFACE_ADDRESS));
+    }
+
+    tmp = GET_REG(EVERGREEN_DACA_BASE);
+    printf("\nDACA: %s\n", tmp & 0x1 ? "Enabled" : "Disabled");
+    if (tmp & 0x1) {
+	printf("DACA en:%08x source:%08x\n", tmp, GET_REG(EVERGREEN_DACA_BASE + AVIVO_DAC_SOURCE_SELECT_OFFSET));
+    }
+
+    tmp = GET_REG(EVERGREEN_DACB_BASE);
+    printf("DACB: %s\n", tmp & 0x1 ? "Enabled" : "Disabled");
+    if (tmp & 0x1) {
+	printf("DACB: en:%08x source:%08x\n", tmp, GET_REG(EVERGREEN_DACB_BASE + AVIVO_DAC_SOURCE_SELECT_OFFSET));
+    }
+    printf("\n");
+
+    if (show_all) {
+	for (i = 0x6250 ; i < 0x626c; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x6400 ; i < 0x641c; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x6420 ; i < 0x647c; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	SHOW_UNKNOWN_REG(0x6590);
+	SHOW_UNKNOWN_REG(0x65b4);
+	SHOW_UNKNOWN_REG(0x65cc);
+
+	for (i = 0x6660 ; i <= 0x66dc; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x6800 ; i <= 0x6840; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x6998 ; i <= 0x69e0; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+
+	for (i = 0x6a00 ; i < 0x6a1c; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x6b00 ; i < 0x6b1c; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	/* 6d00, 6d04 appear to change randomly */
+	for (i = 0x6d08 ; i < 0x6d80; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x6df0 ; i < 0x6e7c; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+
+
+	for (i = 0x6fac ; i <= 0x7014; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x79f0 ; i < 0x7a78; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x7bac ; i <= 0x7c14; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x107ac ; i <= 0x10814; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x113ac ; i <= 0x11414; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x11fac ; i <= 0x12014; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x12bac ; i <= 0x12c14; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+
+	for (i = 0x71f0 ; i <= 0x7220; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x7df0 ; i <= 0x7e20; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x109f0 ; i <= 0x10a20; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x115f0 ; i <= 0x11620; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x121f0 ; i <= 0x12220; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+	for (i = 0x12df0 ; i <= 0x12e20; i+=4)
+	    SHOW_UNKNOWN_REG(i);
+    }
+}
+
 void radeon_cmd_regs(const char *type)
 {
-    #define GET_REG(r) radeon_get(r, #r)
-    #define SHOW_REG(r) printf("%s\t%08x\n", #r, radeon_get(r, #r))
-    #define SHOW_UNKNOWN_REG(r) { tmp = radeon_get(r, "byhand"); printf("%08lx\t%08x (%d)\n", r, tmp, tmp); }
-    #define REG_TYPE_NAME(r) ((r == REG_TYPE_STATIC) ? "static" : ((r == REG_TYPE_SEMI_STATIC) ? "semi-static" : "random"))
-    #define SHOW_STATIC_REG(r) { tmp = get_reg_type(r); printf("%s (%08lx)\t%s\n", get_reg_name(r, ""), r, REG_TYPE_NAME(tmp)); }
-    #define SHOW_REG_DECIMAL(r) printf("%s\t%d (decimal)\n", #r, radeon_get(r, #r))
-    #define SHOW_REG_BITS(r, ...) radeon_show_reg_bits(#r, 0, 0, r, __VA_ARGS__)
-    #define SHOW_MC_REG(r) printf("%s\t%08x\n", #r, radeon_get_indexed(AVIVO_MC_INDEX, AVIVO_MC_DATA, (r | 0x007f0000), #r))
-    #define SHOW_MC_REG_BITS(r, ...) radeon_show_reg_bits(AVIVO_MC_INDEX, AVIVO_MC_DATA, #r, (r | 0x007f0000), __VA_ARGS__)
-    #define SHOW_PCIE_REG(r) printf("%s\t%08x\n", #r, radeon_get_indexed(0x30, 0x34, (r), #r))
-
     int show_all = (strcmp(type, "all") == 0);
     int show_core = (show_all || strstr(type, "core"));
     int show_mc = (show_all || strstr(type, "mc"));
@@ -1056,15 +1215,15 @@ void radeon_cmd_regs(const char *type)
 
     /* Dump all registers that we can read. */
     if (strcmp(type, "all") == 0) {
-        /* Attempting to read 40c8 will bring your machine down hard. */
-        for (i = 0x0000; i < 0x40c8; i += 4)
+        for (i = 0x0000; i < 0x5bc; i += 4)
             SHOW_UNKNOWN_REG(i);
-        /* Ditto 4ff8 and 4ffc.  They may be indexing registers, or they
-         * may just kill your system. */
-        for (i = 0x40cc; i < 0x4ff8; i += 4)
+        for (i = 0x5dfc; i < 0x7170; i += 4)
             SHOW_UNKNOWN_REG(i);
-        /* Reading to the end of the range is also harmful. */
-        for (i = 0x5000; i < 0x7ff8; i += 4)
+        for (i = 0x75a0; i < 0x7764; i += 4)
+            SHOW_UNKNOWN_REG(i);
+        for (i = 0x79a0; i < 0x7b64; i += 4)
+            SHOW_UNKNOWN_REG(i);
+        for (i = 0x7ec0; i < 0x8000; i += 4)
             SHOW_UNKNOWN_REG(i);
 
 	/* dump a bunch of the MC regs */
@@ -1599,6 +1758,8 @@ static int map_radeon_mem(void)
                     ctrl_region = i;
                 else if (device->regions[i].size == 128 * 1024)
                     ctrl_region = i;
+                else if (device->regions[i].size == 256 * 1024)
+                    ctrl_region = i;
                 else if (device->regions[i].size >= 128 * 1024 * 1024)
                     fb_region = i;
             }
@@ -1618,9 +1779,10 @@ static int map_radeon_mem(void)
         fatal("mapping ctrl region\n");
     ctrl_mem = avivo_device->regions[ctrl_region].memory;
 
-    if (pci_device_map_region(avivo_device, fb_region, 1) != 0)
-        fatal("mapping fb region\n");
-    fb_mem = avivo_device->regions[fb_region].memory;
+    if (pci_device_map_region(avivo_device, fb_region, 1) == 0)
+    	fb_mem = avivo_device->regions[fb_region].memory;
+    else
+	fb_mem = NULL;
 
     pci_iterator_destroy(iter);
 
@@ -2094,7 +2256,10 @@ int main(int argc, char *argv[])
 	    return 0;
 	}
         if (strcmp(argv[1], "regs") == 0) {
-            radeon_cmd_regs("default");
+	    if (IS_EG(card_info))
+		eg_cmd_regs("default");
+	    else
+		radeon_cmd_regs("default");
             return 0;
         }
         if (strcmp(argv[1], "i2c") == 0) {
@@ -2116,7 +2281,10 @@ int main(int argc, char *argv[])
             return 0;
         }
         else if (strcmp(argv[1], "regs") == 0) {
-            radeon_cmd_regs(argv[2]);
+	    if (IS_EG(card_info))
+		eg_cmd_regs(argv[2]);
+	    else
+		radeon_cmd_regs(argv[2]);
             return 0;
         }
         else if (strcmp(argv[1], "dumpimg") == 0) {
