@@ -102,9 +102,15 @@ static unsigned int radeon_get_indexed(unsigned long index_offset,
 
 static void radeon_set_indexed(unsigned long index_offset,
                                unsigned long data_offset, unsigned long offset,
-                               const char *name, unsigned int value)
+                               const char *name, unsigned int value, int index_mask)
 {
-    radeon_set(index_offset, "index", offset);
+    unsigned int temp = offset;
+    if (index_mask) {
+	temp = radeon_get(index_offset, "index");
+	temp &= ~index_mask;
+	temp += offset;
+    }
+    radeon_set(index_offset, "index", temp);
     radeon_set(data_offset, name, value);
 }
 
@@ -119,7 +125,7 @@ static void radeon_set_clk(unsigned long offset, const char *name,
                           unsigned int value)
 {
     return radeon_set_indexed(RADEON_CLOCK_CNTL_INDEX, RADEON_CLOCK_CNTL_DATA,
-                              offset | RADEON_PLL_WR_EN, name, value);
+                              offset | RADEON_PLL_WR_EN, name, value, 0xff);
 }
 
 static unsigned int radeon_get_mcind(unsigned long offset, const char *name)
@@ -130,7 +136,7 @@ static unsigned int radeon_get_mcind(unsigned long offset, const char *name)
 static void radeon_set_mcind(unsigned long offset, const char *name,
 			     unsigned int value)
 {
-  return radeon_set_indexed(0x1f8, 0x1fc, offset | (1 << 8), name, value);
+  return radeon_set_indexed(0x1f8, 0x1fc, offset | (1 << 8), name, value, 0);
 }
 
 static void usage(void)
@@ -760,9 +766,11 @@ void radeon_reg_match(const char *pattern)
                 printf("0x%08x (%d)\n", value, value);
             }
         }
-	for (i = 0; i < 0x2f; i++) {
-	  value = radeon_get_mcind(i, "MC");
-	  printf("MC: 0x%02x\t0x%08x\n", i, value);
+	if (pattern[0] == '*') {
+ 	     for (i = 0; i < 0x2f; i++) {
+	         value = radeon_get_mcind(i, "MC");
+	         printf("MC: 0x%02x\t0x%08x\n", i, value);
+	     }
 	}
     }
 }
@@ -962,9 +970,10 @@ static void map_radeon_cntl_mem(void)
         fatal("mapping ctrl region\n");
     ctrl_mem = avivo_device->regions[ctrl_region].memory;
 
-    if (pci_device_map_region(avivo_device, fb_region, 1) != 0)
-        fatal("mapping fb region\n");
-    fb_mem = avivo_device->regions[fb_region].memory;
+    if (pci_device_map_region(avivo_device, fb_region, 1) == 0)
+    	fb_mem = avivo_device->regions[fb_region].memory;
+    else
+	fb_mem = NULL;
 
     pci_iterator_destroy(iter);
 
@@ -1928,7 +1937,12 @@ static void radeon_rom_legacy_list_tables_present(unsigned char *bios, int hdr)
     }
     offset = BIOS16(hdr + 0x48);
     if (offset) {
+	int check_offset = offset;
 	printf("mem config info table\n");
+	while (BIOS8(check_offset++));
+	check_offset += 2;
+	if (check_offset)
+		printf("mem ram reset table %d %x %x\n", check_offset, BIOS8(check_offset -2), BIOS8(check_offset - 1));
     }
     offset = BIOS16(hdr + 0x4a);
     if (offset) {
@@ -2124,6 +2138,32 @@ static void radeon_rom_legacy_miscinfotable(unsigned char *bios, int hdr)
     }
 }
 
+static void radeon_rom_legacy_ram_reset_table(unsigned char *bios, int hdr)
+{
+	int offset = BIOS16(hdr + 0x48);
+	uint32_t val;
+
+	if (!offset)
+		return;
+
+	while (BIOS8(offset++));
+	offset += 2;
+
+	printf("ram reset table %d rev %d and mask %x\n", offset, BIOS8(offset - 1), BIOS8(offset - 2));
+	val = BIOS8(offset);
+	while(val != 0xff) {
+		offset++;
+		if (val == 0x0f) {
+			printf("pwup complete test\n");
+		} else {
+			uint32_t or_mask = BIOS16(offset);
+			offset+=2;
+			printf("rmw SDRAM mask %x %x\n", or_mask, val);
+		}
+		val = BIOS8(offset);
+	}
+}
+
 static void radeon_rom_legacy_mmio_table(unsigned char *bios, int offset)
 {
 	if (!offset)
@@ -2793,6 +2833,8 @@ void radeon_rom_tables(const char * file)
 		printf("-------------\n");
 		radeon_rom_legacy_pllinittable(bios, hdr);
 		printf("-------------\n");
+		radeon_rom_legacy_ram_reset_table(bios, hdr);
+		printf("-------------\n");
 		radeon_rom_legacy_clocks(bios, hdr);
 		printf("-------------\n");
 		radeon_rom_legacy_connectors(bios, hdr);
